<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2016 Amasty (https://www.amasty.com)
 * @package Amasty_Geoip
 */

namespace Amasty\Geoip\Model;


class Geolocation extends \Magento\Framework\Model\AbstractModel
{
    /**
     * @var \Amasty\Geoip\Helper\Data
     */
    public $geoipHelper;

    /**
     * @var \Magento\Framework\App\ResourceConnection
     */
    protected $resource;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var \Magento\Framework\App\Request\Http
     */
    protected $request;

    /**
     * Geolocation constructor.
     * @param \Amasty\Geoip\Helper\Data $geoipHelper
     * @param \Magento\Framework\App\ResourceConnection $resource
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        \Amasty\Geoip\Helper\Data $geoipHelper,
        \Magento\Framework\App\ResourceConnection $resource,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\App\Request\Http $request
    )
    {
        $this->geoipHelper = $geoipHelper;
        $this->resource = $resource;
        $this->scopeConfig = $scopeConfig;
        $this->request = $request;
    }

    /**
     * @param $ip
     * @return $this
     */
    public function locate($ip)
    {
        $countryHeaderConfig = $this->scopeConfig->getValue('amgeoipredirect/restriction/geolocation_header');

        if ($this->request->getHeader($countryHeaderConfig)) {
            $this->setData([
                'country' => $this->request->getHeader($countryHeaderConfig),
                'city' => null,
                'geoip_loc_id' => null,
                'location_id' => null
            ]);
            return $this;
        }

        if (!$this->geoipHelper->isDone()) {
            return $this;
        }

        $longIP = sprintf("%u", ip2long($ip));

        if (empty($longIP)) {
            return $this;
        }

        $db = $this->resource->getConnection();
        $select = $db->select()
            ->from(['l' => $this->resource->getTableName('amasty_geoip_location')])
            ->join(
                ['b' => $this->resource->getTableName('amasty_geoip_block')],
                'b.geoip_loc_id = l.geoip_loc_id',
                []
            )
            ->where("$longIP between b.start_ip_num and b.end_ip_num")
            ->limit(1);

        if ($res = $db->fetchRow($select)) {
            $this->setData($res);
        }

        return $this;
    }
}